@@ -1,6 +1,6 @@
 package My::Module::Meta;
 
-use 5.008;
+use 5.008001;
 
 use strict;
 use warnings;
@@ -19,7 +19,9 @@ sub new {
 
 sub build_requires {
     return +{
-	'Test::More'	=> 0.88,	# Because of done_testing().
+	'Test2::V0'			=> 0,
+	'Test2::Plugin::BailOnFail'	=> 0,
+	'Test2::Tools::LoadModule'	=> 0,
     };
 }
 
@@ -76,7 +78,7 @@ sub requires {
 }
 
 sub requires_perl {
-    return 5.008;
+    return 5.008001;
 }
 
 
@@ -180,7 +182,7 @@ information makes it into F<META.yml>.
 =head1 SUPPORT
 
 Support is by the author. Please file bug reports at
-L<http://rt.cpan.org>, or in electronic mail to the author.
+L<https://rt.cpan.org>, or in electronic mail to the author.
 
 =head1 AUTHOR
 
@@ -188,7 +190,7 @@ Thomas R. Wyant, III F<wyant at cpan dot org>
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright (C) 2019 by Thomas R. Wyant, III
+Copyright (C) 2019-2020 by Thomas R. Wyant, III
 
 This program is free software; you can redistribute it and/or modify it
 under the same terms as Perl 5.10.0. For more details, see the full text
